"""
CLI config command.

Author: Tom Fleet
Created: 24/02/2021
"""

import pathlib

import typer

from pytoil.config import Config

app = typer.Typer(no_args_is_help=True)


# Callback for documentation only
@app.callback()
def config() -> None:
    """
    Interact with pytoil's configuration.

    pytoil's config file is ~/.pytoil.yml.

    Schema:

    username (str): Users GitHub username.

    token (str): Users GitHub personal access token.

    projects_dir (str): Absolute path to where you keep your projects.

    vscode (bool): Whether to use vscode or not.
    """


@app.command()
def show() -> None:
    """
    Display currently set configuration.
    """

    # Get config but don't raise on UNSET
    try:
        config = Config.get()
    except FileNotFoundError:
        typer.secho("No config file yet!", fg=typer.colors.YELLOW)
        typer.echo("Run '$ pytoil init' to automatically generate one.")
    else:
        typer.secho("\nCurrent pytoil config:", fg=typer.colors.BLUE, bold=True)
        typer.echo("")
        config.show()


@app.command()
def set(
    username: str = typer.Option(None, "--username", "-u", help="Set GitHub username."),
    token: str = typer.Option(
        None, "--token", "-t", help="Set Github personal access token."
    ),
    projects_dir: pathlib.Path = typer.Option(
        None,
        "--projects-dir",
        "-p",
        help="Set projects directory.",
        file_okay=False,
        dir_okay=True,
        writable=True,
        readable=True,
        resolve_path=True,
    ),
    vscode: str = typer.Option(
        None, "--vscode", "-v", help="Set pytoil to use vscode.", case_sensitive=False
    ),
) -> None:
    """
    Set a config parameter.

    Examples

    $ pytoil config set --username my_username

    $ pytoil config set --token my_token

    $ pytoil config set --vscode True
    """

    # Get any existing config
    # but don't validate
    try:
        config = Config.get()
    except FileNotFoundError:
        typer.secho("No config file yet!", fg=typer.colors.YELLOW)
        typer.echo("Run '$ pytoil init' to automatically generate one.")
    else:

        # I'm not keen on this, it feels messy
        if username:
            config.username = username
        elif token:
            config.token = token
        elif projects_dir:
            config.projects_dir = projects_dir
        elif vscode:  # pragma: no cover
            # If we use vscode as a boolean option it doesn't quite work right
            # also for some reason coverage doesn't recognise this as being called
            # during tests but it is in tests/cli/test_config.py so we exclude it
            if vscode.lower() == "true":
                config.vscode = True
            elif vscode.lower() == "false":
                config.vscode = False
            else:
                raise typer.BadParameter("vscode must be a boolean value.")

        # Write the updated config
        config.write()

        typer.secho("\nConfig updated successfully!", fg=typer.colors.GREEN)
        typer.secho("\nNew Config:\n", fg=typer.colors.BLUE, bold=True)
        config.show()



"""
Main CLI module.

Author: Tom Fleet
Created: 24/02/2021
"""

import pathlib

import typer

from pytoil import __version__
from pytoil.cli import config, project, show, sync
from pytoil.config import CONFIG_PATH, Config
from pytoil.exceptions import InvalidConfigError

# Add all the subcommands
app = typer.Typer(name="pytoil", no_args_is_help=True)
app.add_typer(config.app, name="config")
app.add_typer(project.app, name="project")
app.add_typer(sync.app, name="sync")
app.add_typer(show.app, name="show")


def version_callback(value: bool) -> None:
    if value:
        typer.echo(f"pytoil version: {__version__}")
        raise typer.Exit()


# Callback for documentation only
@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "--version",
        "-V",
        help="Display pytoil version.",
        callback=version_callback,
        is_eager=True,
        show_default=False,
    )
) -> None:
    """
    Helpful CLI to automate the development workflow.

    - Create and manage your local and remote projects

    - Build projects from cookiecutter templates.

    - Easily create/manage virtual environments.

    - Minimal configuration required.
    """


@app.command()
def init() -> None:
    """
    Initialise pytoil.

    Will guide user through the config setup
    interactively.
    """

    typer.secho("Checking for config file...\n", fg=typer.colors.BLUE, bold=True)

    # Make config file
    if not CONFIG_PATH.exists():
        typer.secho("No config file found!\n", fg=typer.colors.YELLOW)
        typer.echo("Creating fresh config file...\n")
        CONFIG_PATH.touch()
        default_config = Config()
        default_config.write()

        username: str = typer.prompt("GitHub username")
        token: str = typer.prompt("GitHub personal access token")
        projects_dir: str = typer.prompt("Absolute path to your projects directory")
        vscode: str = typer.prompt("Use VSCode to open projects with? [True|False]")

        projects_dir_path = pathlib.Path(projects_dir)

        if vscode.lower() == "true":
            vscode_bool = True
        elif vscode.lower() == "false":
            vscode_bool = False
        else:
            raise typer.BadParameter("VSCode must be a boolean value")

        user_config = Config(
            username=username,
            token=token,
            projects_dir=projects_dir_path,
            vscode=vscode_bool,
        )
        user_config.write()

        typer.secho("Config written, you're good to go!", fg=typer.colors.GREEN)
    else:
        try:
            config = Config.get()
            config.validate()
        except InvalidConfigError:
            typer.secho(
                "Something's wrong in the config file, please check it.\n",
                fg=typer.colors.RED,
            )
            typer.echo("If in doubt, simply delete it and run '$ pytoil init' again :)")
            raise typer.Abort()
        else:
            typer.secho("You're all set!", fg=typer.colors.GREEN)
